package cn.iocoder.yudao.module.system.controller.admin.bannerPop;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.system.controller.admin.bannerPop.vo.BannerPopBackVO;
import cn.iocoder.yudao.module.system.controller.admin.bannerPop.vo.BannerPopUpdateReqVO;
import cn.iocoder.yudao.module.system.controller.admin.bannerPop.vo.BannerPopCreateReqVO;
import cn.iocoder.yudao.module.system.controller.admin.bannerPop.vo.BannerPopQueryVO;
import cn.iocoder.yudao.module.system.convert.bannerPop.BannerPopConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.bannerPop.BannerPopDO;
import cn.iocoder.yudao.module.system.service.bannerPop.BannerPopService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;


@Validated
@RestController
@Api(tags = "管理后台 - 广告弹窗")
@RequestMapping("/ecw/banner-pop")
public class BannerPopController {

    @Resource
    private BannerPopService bannerPopService;

    @PostMapping("/create")
    @ApiOperation("创建广告弹窗")
    @PreAuthorize("@ss.hasPermission('ecw:banner-pop:create')")
    public CommonResult<Long> createBannerPop(@Valid @RequestBody BannerPopCreateReqVO createReqVO) {
        return success(bannerPopService.createBannerPop(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新广告弹窗")
    @PreAuthorize("@ss.hasPermission('ecw:banner-pop:update')")
    public CommonResult<Boolean> updateBannerPop(@Valid @RequestBody BannerPopUpdateReqVO updateReqVO) {
        bannerPopService.updateBannerPop(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除广告弹窗")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:banner-pop:delete')")
    public CommonResult<Boolean> deleteBannerPop(@RequestParam("id") Long id) {
        bannerPopService.deleteBannerPop(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得广告弹窗")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:banner-pop:query')")
    public CommonResult<BannerPopBackVO> getBannerPop(@RequestParam("id") Long id) {
        BannerPopDO bannerPop = bannerPopService.getBannerPop(id);
        return success(BannerPopConvert.INSTANCE.convert(bannerPop));
    }

    @GetMapping("/list")
    @ApiOperation("获得广告弹窗列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:banner-pop:query')")
    public CommonResult<List<BannerPopBackVO>> getBannerPopList(@RequestParam("ids") Collection<Long> ids) {
        List<BannerPopDO> list = bannerPopService.getBannerPopList(ids);
        return success(BannerPopConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得广告弹窗分页")
    @PreAuthorize("@ss.hasPermission('ecw:banner-pop:query')")
    public CommonResult<PageResult<BannerPopBackVO>> getBannerPopPage(@Valid BannerPopQueryVO query, PageVO page) {
        if (Objects.isNull(query)){
            // 保证后面查询条件判断不报空指针
            query = new BannerPopQueryVO();
        }
        PageResult<BannerPopDO> pageResult = bannerPopService.getBannerPopPage(query, page, 2);
        return success(BannerPopConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出广告弹窗 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:banner-pop:export')")
    @OperateLog(type = EXPORT)
    public void exportBannerPopExcel(@Valid BannerPopQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BannerPopDO> list = bannerPopService.getBannerPopList(query);
        // 导出 Excel
        List<BannerPopBackVO> datas = BannerPopConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "广告弹窗.xls", "数据", BannerPopBackVO.class, datas);
    }
}
