package cn.iocoder.yudao.module.system.service.bannerPop;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.bannerPop.vo.BannerPopCreateReqVO;
import cn.iocoder.yudao.module.system.controller.admin.bannerPop.vo.BannerPopQueryVO;
import cn.iocoder.yudao.module.system.controller.admin.logger.vo.loginlog.LoginLogPageReqVO;
import cn.iocoder.yudao.module.system.convert.bannerPop.BannerPopConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.bannerPop.BannerPopDO;
import cn.iocoder.yudao.module.system.dal.dataobject.notice.NoticeDO;
import cn.iocoder.yudao.module.system.dal.mysql.bannerPop.BannerPopMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.module.system.controller.admin.bannerPop.vo.BannerPopUpdateReqVO;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.BANNER_POP_NOT_EXISTS;

/**
 * 广告弹窗 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BannerPopServiceImpl extends AbstractService<BannerPopMapper, BannerPopDO> implements BannerPopService {

    @Resource
    private BannerPopMapper bannerPopMapper;

    @Override
    public Long createBannerPop(BannerPopCreateReqVO createReqVO) {
        // 插入
        BannerPopDO bannerPop = BannerPopConvert.INSTANCE.convert(createReqVO);
        bannerPopMapper.insert(bannerPop);
        // 返回
        return bannerPop.getId();
    }

    @Override
    public void updateBannerPop(BannerPopUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBannerPopExists(updateReqVO.getId());
        // 更新
        BannerPopDO updateObj = BannerPopConvert.INSTANCE.convert(updateReqVO);
        bannerPopMapper.updateById(updateObj);
    }

    @Override
    public void deleteBannerPop(Long id) {
        // 校验存在
        this.validateBannerPopExists(id);
        // 删除
        bannerPopMapper.deleteById(id);
    }

    private void validateBannerPopExists(Long id) {
        if (bannerPopMapper.selectById(id) == null) {
            throw exception(BANNER_POP_NOT_EXISTS);
        }
    }

    @Override
    public BannerPopDO getBannerPop(Long id) {
        return bannerPopMapper.selectById(id);
    }

    @Override
    public List<BannerPopDO> getBannerPopList(Collection<Long> ids) {
        return bannerPopMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BannerPopDO> getBannerPopPage(BannerPopQueryVO query, PageVO page, Integer userType) {
        LambdaQueryWrapper<BannerPopDO> queryWrapper = new LambdaQueryWrapper<>();
        if (Objects.nonNull(userType) && userType == 1) {
            // 会员端访问，只访问状态开启的和未过期的
            queryWrapper.eq(BannerPopDO::getStatus, 0);
            queryWrapper.apply("((validate_start IS NOT NULL AND validate_end IS NULL AND validate_start < now())\n" +
                    "\t\tOR \n" +
                    "\t\t( validate_end IS NOT NULL AND validate_start IS NULL AND validate_end > now())\n" +
                    "\t\tor\n" +
                    "\t  (validate_start IS NOT NULL and validate_start < now() and validate_end IS NOT NULL and validate_end > now() ) )\t");
        }
        queryWrapper.eq(Objects.nonNull(query.getStatus()), BannerPopDO::getStatus, query.getStatus());
        queryWrapper.between(Objects.nonNull(query.getBeginCreateTime()) && Objects.nonNull(query.getEndCreateTime()), BannerPopDO::getCreateTime, query.getBeginCreateTime(), query.getEndCreateTime());
        queryWrapper.orderByDesc(BannerPopDO::getUpdateTime);
        return bannerPopMapper.selectPage(page, queryWrapper);
    }

    @Override
    public List<BannerPopDO> getBannerPopList(BannerPopQueryVO query) {
        return bannerPopMapper.selectList(query);
    }
}
