package cn.iocoder.yudao.module.system.service.sms;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.sms.core.client.impl.sendchamp.SendChampSmsClient;
import cn.iocoder.yudao.framework.sms.core.enums.ReceiveStatusEnum;
import cn.iocoder.yudao.framework.sms.core.enums.SendChampReceiveStatusEnum;
import cn.iocoder.yudao.module.system.controller.admin.sms.vo.log.SmsLogExportReqVO;
import cn.iocoder.yudao.module.system.controller.admin.sms.vo.log.SmsLogPageReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.sms.SmsLogDO;
import cn.iocoder.yudao.module.system.dal.dataobject.sms.SmsTemplateDO;
import cn.iocoder.yudao.module.system.dal.mysql.sms.SmsLogMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.enums.sms.SmsReceiveStatusEnum;
import cn.iocoder.yudao.module.system.enums.sms.SmsSendStatusEnum;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * 短信日志 Service 实现类
 *
 * @author zzf
 */
@Slf4j
@Service
public class SmsLogServiceImpl implements SmsLogService {

    @Resource
    private SmsLogMapper smsLogMapper;

    @Override
    public Long createSmsLog(String mobile, Long userId, Integer userType, Boolean isSend, SmsTemplateDO template,
                             String templateContent, Map<String, Object> templateParams, String apiTemplateId, Long smsLogId) {
        SmsLogDO.SmsLogDOBuilder logBuilder = SmsLogDO.builder();
        // 根据是否要发送，设置状态
        logBuilder.sendStatus(Objects.equals(isSend, true) ? SmsSendStatusEnum.INIT.getStatus()
                : SmsSendStatusEnum.IGNORE.getStatus());
        // 设置手机相关字段
        logBuilder.mobile(mobile).userId(userId).userType(userType);
        // 设置模板相关字段
        logBuilder.templateId(template.getId()).templateCode(template.getCode()).templateType(template.getType());
        logBuilder.templateContent(templateContent).templateParams(templateParams)
                .apiTemplateId(apiTemplateId);
        // 设置渠道相关字段
        logBuilder.channelId(template.getChannelId()).channelCode(template.getChannelCode());
        // 设置接收相关字段
        logBuilder.receiveStatus(SmsReceiveStatusEnum.INIT.getStatus());
        // 重发短信的原短信日志id
        logBuilder.smsLogId(smsLogId);
        // 默认重发次数是0
        logBuilder.num(0);
        // 插入数据库
        SmsLogDO logDO = logBuilder.build();
        smsLogMapper.insert(logDO);
        return logDO.getId();
    }

    /**
     * 创建短信日志
     *
     * @param mobile          手机号
     * @param userId          用户编号
     * @param userType        用户类型
     * @param isSend          是否发送
     * @param template        短信模板
     * @param templateContent 短信内容
     * @param templateParams  短信参数
     * @param apiTemplateId   短信 API 的模板编号
     * @param smsLogId        重发短信时的原短信日志id
     * @param nodeId          节点id
     * @param nodeTemplateSn  节点模板序列号
     * @param messageType     发送类型
     * @return 发送日志编号
     */
    @Override
    public Long createSmsLogV2(String mobile, Long userId, Integer userType, Boolean isSend,
                               SmsTemplateDO template, String templateContent, Map<String, Object> templateParams, String apiTemplateId, Long smsLogId,
                               Long nodeId, Integer nodeTemplateSn, Integer messageType) {
        SmsLogDO.SmsLogDOBuilder logBuilder = SmsLogDO.builder();
        // 根据是否要发送，设置状态
        logBuilder.sendStatus(Objects.equals(isSend, true) ? SmsSendStatusEnum.INIT.getStatus()
                : SmsSendStatusEnum.IGNORE.getStatus());
        // 设置手机相关字段
        logBuilder.mobile(mobile).userId(userId).userType(userType);
        // 设置模板相关字段
        logBuilder.templateId(template.getId()).templateCode(template.getCode()).templateType(template.getType());
        logBuilder.templateContent(templateContent).templateParams(templateParams)
                .apiTemplateId(apiTemplateId);
        // 设置渠道相关字段
        logBuilder.channelId(template.getChannelId()).channelCode(template.getChannelCode());
        // 设置接收相关字段
        logBuilder.receiveStatus(SmsReceiveStatusEnum.INIT.getStatus());
        // 重发短信的原短信日志id
        logBuilder.smsLogId(smsLogId);
        // 默认重发次数是0
        logBuilder.num(0);
        // 节点id
        logBuilder.nodeId(nodeId);
        // 节点模板序列号
        logBuilder.nodeTemplateSn(nodeTemplateSn);
        // 默认重发次数是0
        logBuilder.messageType(messageType);
        // 发送类型
        SmsLogDO logDO = logBuilder.build();
        smsLogMapper.insert(logDO);
        return logDO.getId();
    }

    @Override
    public void updateSmsSendResult(Long id, Integer sendCode, String sendMsg,
                                    String apiSendCode, String apiSendMsg,
                                    String apiRequestId, String apiSerialNo) {
        SmsSendStatusEnum sendStatus = CommonResult.isSuccess(sendCode) ?
                SmsSendStatusEnum.SUCCESS : SmsSendStatusEnum.FAILURE;
        smsLogMapper.updateById(SmsLogDO.builder().id(id).sendStatus(sendStatus.getStatus())
                .sendTime(new Date()).sendCode(sendCode).sendMsg(sendMsg)
                .apiSendCode(apiSendCode).apiSendMsg(apiSendMsg)
                .apiRequestId(apiRequestId).apiSerialNo(apiSerialNo).build());
    }

    @Override
    public void updateSmsReceiveResult(Long id, Boolean success, Date receiveTime,
                                       String apiReceiveCode, String apiReceiveMsg) {
        SmsReceiveStatusEnum receiveStatus = Objects.equals(success, true) ?
                SmsReceiveStatusEnum.SUCCESS : SmsReceiveStatusEnum.FAILURE;
        smsLogMapper.updateById(SmsLogDO.builder().id(id).receiveStatus(receiveStatus.getStatus())
                .receiveTime(receiveTime).apiReceiveCode(apiReceiveCode).apiReceiveMsg(apiReceiveMsg).build());
    }

    @Override
    public PageResult<SmsLogDO> getSmsLogPage(SmsLogPageReqVO pageReqVO) {
        return smsLogMapper.selectPage(pageReqVO);
    }

    @Override
    public List<SmsLogDO> getSmsLogList(SmsLogExportReqVO exportReqVO) {
        return smsLogMapper.selectList(exportReqVO);
    }

    @Override
    public List<SmsLogDO> listByIds(Collection<Long> ids) {
        return smsLogMapper.selectList(new LambdaQueryWrapper<SmsLogDO>().in(SmsLogDO::getId, ids));
    }

    @Override
    public SmsLogDO getById(Long smsLogId) {
        return smsLogMapper.selectById(smsLogId);
    }

    @Override
    public void updateBatchSmsLog(List<SmsLogDO> resendSmsLogList) {
        smsLogMapper.updateBatch(resendSmsLogList);
    }

    @Override
    public List<SmsLogDO> selectReceiveList(Integer time) {
        return smsLogMapper.selectReceiveList(time);
    }

    @Override
    public void updateSendChampReceive(SendChampSmsClient.SmsReceiveStatus smsReceiveStatus) {
        if (SendChampReceiveStatusEnum.DELIVERED.getCode().equals(smsReceiveStatus.getStatus())) {
            List<SmsLogDO> smsLogs = smsLogMapper.selectList(new SmsLogExportReqVO().setApiSerialNo(smsReceiveStatus.getReference()));
            if (smsLogs != null) {
                SmsLogDO smsLogDO = smsLogs.get(0);
                smsLogDO.setReceiveStatus(ReceiveStatusEnum.Receive_STATUS_10.getValue());
                smsLogDO.setReceiveTime(new Date());
                smsLogMapper.updateById(smsLogDO);
            }
        }
    }
}
